#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

# with open('HISTORY.md') as history_file:
#    history = history_file.read()

# install requirements
requirements = ['scipy', 'tdqm', 'python_speech_features', 'numpy', 'soundfile']

setup_requirements = []

test_requirements = []

setup(
    author="yinochaos",
    author_email='pspcxl@163.com',
    python_requires='>=3.5',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="datasets for easy machine learning use",
    entry_points={
        'console_scripts': [
            'datasets=datasets.cli:main',
        ],
    },
    install_requires=requirements,
    license="Apache Software License 2.0",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    keywords='ml-dataset',
    name='ml-dataset',
    packages=find_packages(include=['datasets', 'datasets.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/yinochaos/datasets',
    version='0.0.16',
    zip_safe=False,
)
